/*********************************************************************
 * Licensed Materials - PropertyImpl of IBM                              *
 *                                                                   *
 * 5724-B88                                                          *
 *                                                                   *
 *  Copyright IBM Corp.  2001, 2002    All Rights Reserved.         *
 *                                                                   *
 * US Government Users Restricted Rights - Use, duplication or       *
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp. *
 *********************************************************************/

package com.ibm.websphere.query.base;

import java.util.ArrayList;
import java.util.List;

/**
	<P>The Condition class represents a condition in a predicate in the query framework. It is 
	derived from PredicateBase.
	
	<P>This class contains left operand, operator, and right operands, to describe a typical 
	condition in a query. For example, a simple condition "age > 30" will be decomposed	as 
	left operand = Attribute("age"), operator = Operator(">"), and right operand = Value("30")
 */
public class Condition extends PredicateBase implements ICondition
{
  private IOperator operator = null;
  private ILeftOperand leftOperand = null;
  private List rightOperands = new ArrayList();

  /**
  	Creates a new Condition instance, default constructor.
   */
  public Condition()
  {
    super();
  }

  /**
  	Creates a new Condition instance with the given left operand object.
   */
  public Condition(ILeftOperand newLeftOperand)
  {
    leftOperand = newLeftOperand;
  }

  /**
  	Creates a new Condition instance with the given left operand, and operator.
   */
  public Condition(ILeftOperand newLeftOperand, IOperator newOperator)
  {
    leftOperand = newLeftOperand;
    operator = newOperator;
  }

  /**
  	Creates a new Condition instance with the given left operand, operator, and an array of
  	right operands.
   */
  public Condition(ILeftOperand newLeftOperand, IOperator newOperator, IRightOperand[] newRightOperands)
  {
    leftOperand = newLeftOperand;
    operator = newOperator;
    setRightOperands(newRightOperands);
  }
  /**
  	Creates a new Condition instance with the given left operand, operator, and right operand.
   */
  public Condition(ILeftOperand newLeftOperand, IOperator newOperator, IRightOperand newRightOperand)
  {
    leftOperand = newLeftOperand;
    operator = newOperator;
    setRightOperand(newRightOperand);
  }
  /**
  	Adds a right operand to the right operand vector.
   */
  public void addRightOperand(IRightOperand r)
  {
    rightOperands.add(r);
  }
  /**
  	Builds a query string for the condition.
   */
  public String buildString(ISelectQueryCallback builder) throws QueryException
  {
    return builder.buildCondition(this);
  }
  public void buildString(ISelectQueryCallback builder, StringBuffer sb) throws QueryException
  {
    builder.buildCondition(this, sb);
  }
  /**
  	Tests for equality with the given object.
   */
  public boolean equals(Object o)
  {
    boolean eq = super.equals(o);

    if ((eq == true) && (o instanceof Condition))
    {
      Condition c = (Condition) o;

      if (leftOperand != null)
      {
        eq = leftOperand.equals(c.leftOperand);
      }
      else
      {
        eq = (c.leftOperand == null);
      }

      if (eq)
      {
        if (operator != null)
        {
          eq = operator.equals(c.operator);
        }
        else
        {
          eq = (c.operator == null);
        }
      }

      if (eq)
      {
        eq = rightOperands.equals(c.rightOperands);
      }
    }
    else
    {
      eq = false;
    }

    return eq;
  }
  /**
  	Returns the left operand.
   */
  public ILeftOperand getLeftOperand()
  {
    return leftOperand;
  }
  /**
  	Returns the operator.
   */
  public IOperator getOperator()
  {
    return operator;
  }
  /**
  	Returns the right operands as an array.
   */
  public IRightOperand[] getRightOperands()
  {
    IRightOperand ra[] = new IRightOperand[rightOperands.size()];

    for (int i = 0; i < rightOperands.size(); i++)
    {
      ra[i] = (IRightOperand) (rightOperands.get(i));
    }

    return ra;
  }
  /**
  	Computes a hash code for the condition.
   */
  public int hashCode()
  {
    int hashCode = super.hashCode();

    if (leftOperand != null)
      hashCode = leftOperand.hashCode();

    if (operator != null)
      hashCode = hashCode ^ operator.hashCode();

    for (int i = 0; i < rightOperands.size(); i++)
    {
      hashCode = hashCode ^ rightOperands.get(i).hashCode();
    }

    return hashCode;
  }
  /**
  	Sets the left operand of the condition.
   */
  public void setLeftOperand(ILeftOperand newLeftOperand)
  {
    leftOperand = newLeftOperand;
  }
  /**
  	Sets the operator of the condition.
   */
  public void setOperator(IOperator newOperator)
  {
    operator = newOperator;
  }
  /**
  	Sets the right operand of the condition.
   */
  public void setRightOperand(IRightOperand r)
  {
    rightOperands.clear();
    rightOperands.add(r);
  }
  /**
  	Sets the right operands of the condition.
   */
  public void setRightOperands(IRightOperand[] newRightOperands)
  {
    rightOperands.clear();

    for (int i = 0; i < newRightOperands.length; i++)
    {
      rightOperands.add(newRightOperands[i]);
    }
  }
}
