/*********************************************************************
 * Licensed Materials - Property of IBM                              *
 *                                                                   *
 * 5724-B88                                                          *
 *                                                                   *
 *  Copyright IBM Corp.  2001, 2002    All Rights Reserved.         *
 *                                                                   *
 * US Government Users Restricted Rights - Use, duplication or       *
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp. *
 *********************************************************************/

package com.ibm.websphere.query.base;

import java.io.Serializable;
/**
<P>The ISelectQueryCallback interface is intended to facilitate query string generation based
on the domain specific requirements.

<P>A class implements this interface to indicate that the instances of that class can produce
a predicate string, an order by string, and other sub strings by introspecting the query
objects in the framework. The query domain expert should resolve attribute mapping and 
syntax conversion in such a callback class so the meaningful query strings can be generically
generated. That means the business developers for a domain can easily use the query 
framework without the knowledge of framework architecture or query callback implementation.
*/
public interface ISelectQueryCallback extends Serializable {
		
	public static final String MULTIVALUE_PROPERTY_NAMES = "multiValuePropertyNames";
	public static final String TABLE_NAME = "tableName";
	public static final String SELECT_ATTRIBUTES = "selectAttributes";
	public static final String TABLE_MAP = "tableMap";
	public static final String LAST_TABLE_ALIAS_INTEGER = "lastTableAliasInteger";
	public static final String PREDICATE_IS_NULL = "predicateIsNull";
	public static final String JOIN_CONDITIONS = "joinConditions";
	public static final String JOIN_TABLES = "joinTables";
	public static final String JOINS = "joins";
	public static final String NAME_MAPPING = "nameMapping";
	public static final String TYPE_MAPPING = "typeMapping";
	
    /**
    	Returns the string representation for the given attribute. Domain specific attribute
    	mapping can be used here to generate meaningful strings.
     */
    String buildAttribute(IAttribute attribute) throws QueryException;
    
	void buildAttribute(IAttribute attribute, StringBuffer sb) throws QueryException;
	
    /**
    	<P>Returns the string representation for the given condition. Domain specific condition
    	syntax can be applied here to generate meaningful strings.
    	
    	<P>This method should define string generation for all possible operators in terms of text for
    	operators and notatation for attributes (infix, postfix or prefix). A sample code in the 
    	builder callback could look like:
     
    	<P>if (operator is GT)
    		...
     	<P>elseif (operator is BTW)
    		...
     	<P>elseif (operator is MVALL)
    		...
     	<P>elseif ......
     
     */
    String buildCondition(ICondition Condition) throws QueryException;
    
	void buildCondition(ICondition Condition, StringBuffer sb) throws QueryException;
	
    /**
    	Returns the string representation for the given function. Domain specific function syntax
    	can be applied here to generate meaningful strings.
     */
    String buildFunction(IFunction function) throws QueryException;
    
	void buildFunction(IFunction function, StringBuffer sb) throws QueryException;
	
    /**
    	Returns the string representation for the given operator. Domain specific operator syntax
    	can be applied here to generate meaningful strings.
     */
    String buildArithmeticCondition(IArithmeticCondition arithmeticCondition) throws QueryException;
    
	void buildArithmeticCondition(IArithmeticCondition arithmeticCondition, StringBuffer sb) throws QueryException;
	
    /**
    	Returns the string representation for the given operator. Domain specific operator syntax
    	can be applied here to generate meaningful strings.
     */
    String buildOperator(IOperator operator) throws QueryException;
    
	void buildOperator(IOperator operator, StringBuffer sb) throws QueryException;
	
    /**
    	Returns the string representation for the given order by clause. Domain specific order by
    	syntax can be applied here to generate meaningful strings.
     */
    String buildOrderBy(IOrderBy orderBy) throws QueryException;
    
	void buildOrderBy(IOrderBy orderBy, StringBuffer sb) throws QueryException;
	
    /**
    	Returns the string representation for the order by part of the given select query. Domain 
    	specific order by syntax can be applied here to generate meaningful strings.
     */
    String buildOrderBy(ISelectQuery selectQuery) throws QueryException;
    
	void buildOrderBy(ISelectQuery selectQuery, StringBuffer sb) throws QueryException;
	
    /**
    	Returns the string representation for the given order by expression. Domain specific order
    	by expression syntax can be applied here to generate meaningful strings.
     */
    String buildOrderByExpression(IOrderByExpression orderByExpression) throws QueryException;
    
	void buildOrderByExpression(IOrderByExpression orderByExpression, StringBuffer sb) throws QueryException;
	
    /**
    	Returns the string representation for the given limit clause. Domain specific limit
    	syntax can be applied here to generate meaningful strings.
     */
    String buildLimit(ILimit limit) throws QueryException;
    
	void buildLimit(ILimit limit, StringBuffer sb) throws QueryException;
	
    /**
    	Returns the string representation for the limit part of the given select query. Domain 
    	specific limit syntax can be applied here to generate meaningful strings.
     */
    String buildLimit(ISelectQuery selectQuery) throws QueryException;
    
	void buildLimit(ISelectQuery selectQuery, StringBuffer sb) throws QueryException;
	
    /**
    	Returns the string representation for the given predicate. Domain specific predicate syntax
    	can be applied here to generate meaningful strings.
     */
    String buildPredicate(IPredicate predicate) throws QueryException;
    
	void buildPredicate(IPredicate predicate, StringBuffer sb) throws QueryException;
	
    /**
    	Returns the string representation for the predicate part of the given query. Domain specific 
    	predicate syntax can be applied here to generate meaningful strings.
     */
    String buildPredicate(IQuery query) throws QueryException;
    
	void buildPredicate(IQuery query, StringBuffer sb) throws QueryException;
	
    /**
    	Returns the string representation for the given query. Domain specific query syntax
    	can be applied here to generate meaningful strings.
     */
    String buildQuery(IQuery query) throws QueryException;
    /**
    	Returns the string representation for the given select query. Domain specific query syntax
    	can be applied here to generate meaningful strings.
     */
    String buildSelectQuery(ISelectQuery selectQuery) throws QueryException;
    
	void buildSelectQuery(ISelectQuery selectQuery, StringBuffer sb) throws QueryException;
	
    /**
    	Returns the string representation for the given value. Domain specific value syntax
    	can be applied here to generate meaningful strings.
     */
    String buildValue(IValue value) throws QueryException;
    
	void buildValue(IValue value, StringBuffer sb) throws QueryException;
	
	/**
	 * Returns the string representation for the given delete query.
	 */
	void buildDeleteQuery(IDeleteQuery selectQuery, StringBuffer sb) throws QueryException;  
	/**
	 * Returns the string representation for the given table.
	 */
	void buildTable(ITable table, StringBuffer sb) throws QueryException;
	/**
	 * Returns the string representation for the given select.
	 */
	void buildSelect(ISelect select, StringBuffer sb) throws QueryException;
	/**
	 * Returns the string representation for the given from.
	 */
	void buildFrom(IFrom from, StringBuffer sb) throws QueryException;
	/**
	 * Returns the string representation for the given from.
	 */
	void buildFrom(ISelectQuery selectQuery, StringBuffer sb) throws QueryException;
	/**
	 * Returns the string representation for the given from.
	 */
	void buildFrom(IDeleteQuery deleteQuery, StringBuffer sb) throws QueryException;
	/**
	 * Returns the string representation for the given join.
	 */
	void buildJoin(IJoin join, StringBuffer sb) throws QueryException;
}