/*
 * SimpleValue.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr;

import java.util.Calendar;

/**
 * A <code>BooleanValue</code> provides an implementation
 * of the <code>Value</code> interface representing a boolean value.
 *
 * @author Stefan Guggisberg
 */
public class BooleanValue extends BaseValue {

    public static final int TYPE = PropertyType.BOOLEAN;

    private Boolean bool = null;

    /**
     * Constructs a <code>BooleanValue</code> object representing a boolean.
     *
     * @param bool the boolean this <code>BooleanValue</code> should represent
     */
    public BooleanValue(Boolean bool) {
        super(TYPE);
        this.bool = bool;
    }

    /**
     * Constructs a <code>BooleanValue</code> object representing a boolean.
     *
     * @param bool the boolean this <code>BooleanValue</code> should represent
     */
    public BooleanValue(boolean bool) {
        super(TYPE);
        this.bool = Boolean.valueOf(bool);
    }

    /**
     * Returns a new <code>BooleanValue</code> initialized to the value
     * represented by the specified <code>String</code>.
     * @param s the string to be parsed.
     * @return a newly constructed <code>BooleanValue</code> representing the
     *         the specified value.
     */
    public static BooleanValue valueOf(String s) {
        return new BooleanValue(Boolean.valueOf(s));
    }


    //----------------------------------------------------------------< Value >
    /**
     * @see Value#getDate
     */
    public Calendar getDate() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        throw new ValueFormatException("conversion to date failed: inconvertible types");
    }

    /**
     * @see Value#getLong
     */
    public long getLong() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        throw new ValueFormatException("conversion to long failed: inconvertible types");
    }

    /**
     * @see Value#getBoolean
     */
    public boolean getBoolean() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (bool != null) {
            return bool.booleanValue();
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getDouble
     */
    public double getDouble() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        throw new ValueFormatException("conversion to double failed: inconvertible types");
    }

    /**
     * @see Value#getString
     */
    public String getString() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (bool != null) {
            return bool.toString();
        } else {
            throw new ValueFormatException("empty value");
        }
    }
    
    public boolean equals(Object obj)
    {
      boolean equals = false;
      if (obj instanceof BooleanValue)
      {
        BooleanValue boolValue = (BooleanValue)obj;
        if (boolValue.bool != null && bool != null)
        {
          equals = boolValue.bool.equals(bool);
        }
        else if (boolValue.bool == null && bool == null)
        {
          equals = true;
        }
      }
      return equals;
    }
    
    public int hashCode()
    {
      int hashCode = 37;
      if (bool != null)
      {
        hashCode = bool.hashCode();
      }
      return hashCode;
    }
}
