/*
 * Item.java
 *
 * This file is part of the specification for the
 * Java Content Repository API.
 */
package javax.jcr;

import javax.jcr.access.AccessDeniedException;
import javax.jcr.lock.Lock;

/**
 * The <code>Item</code> is the base interface of <code>{@link Node}</code>
 * and <code>{@link Property}</code>.
 *
 * @author Peeter Piegaze
 * @author Stefan Guggisberg
 */
public interface Item {

    /**
     * Returns the path to this <code>Item</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * In level 1 implementations
     * there will be only one path to this <code>Item</code> (since hard links
     * are not supported at level 1) and this method returns that path.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * In level 2 implementations (those that <i>do</i> support hard links) this
     * <code>Item</code> may have more than one path.  In that case,
     * this method returns one of those paths. The mechanism by which the
     * implementation chooses which path to return is left unspecified, the only
     * requirement being that for a given <code>Item</code> this
     * method always returns the same path.  To get all paths to this
     * <code>Item</code> use <code>{@link #getPaths()}</code>.
     *
     * @return the path (or one of the paths, in case of hard links) of this
     * <code>Item</code>.
     */
    public String getPath();

//    /**
//     * Returns all paths to this <code>Item</code>.
//     * <p>
//     * <b>Level 1:</b>
//     * <p>
//     * Returns the path to this <code>Item</code>. This is the same
//     * path returned by <code>{@link #getPath()}</code>.
//     * <p>
//     * <b>Level 2:</b>
//     * <p>
//     * Returns all paths to this <code>Item</code>.
//     *
//     * @return A <code>StringIterator</code> of strings: all paths that lead to this
//     * <code>Item</code>.
//     */
//    public StringIterator getPaths();

    /**
     * Returns the name of this <code>Item</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * The name is the last item in the path. If this <code>Item</code>
     * is the root node of the repository (i.e. if
     * <code>this.getDepth() == 0</code>), an empty string will be returned.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * Returns one of the names of this <code>Item</code>. The name returned
     * is the last item in the path returned by <code>{@link #getPath()}</code>.
     *
     * @return the (or a) name of this <code>Item</code> or an empty string
     * if this <code>Item</code> is the root node.
     */
    public String getName();

    /**
     * Returns the ancestor of the specified absolute degree.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * An ancestor of
     * absolute degree <i>x</i> is the <code>Item</code> that is <i>x</i>
     * levels down along the path from the root node to <i>this</i>
     * <code>Item</code>.
     * <ul>
     * <li><i>degree</i> = 0 returns the root node.
     * <li><i>degree</i> = 1 returns the child of the root node along the path
     * to <i>this</i> <code>Item</code>.
     * <li><i>degree</i> = 2 returns the grandchild of the root node along the
     * path to <i>this</i> <code>Item</code>.
     * <li>And so on to <i>degree</i> = <i>n</i>, where <i>n</i> is the depth
     * of <i>this</i> <code>Item</code>, which returns <i>this</i>
     * <code>Item</code> itself.
     * </ul>
     * If <i>degree</i> &gt; <i>n</i> is specified then a
     * <code>ItemNotFoundException</code> is thrown.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * Same behaviour as level 1, but if multiple paths exist to this
     * <code>Item</code> then the path used is the same one that is returned
     * by <code>{@link #getPath()}</code>.
     *
     * @param degree An integer, 0 &lt;= <i>degree</i> &lt;= <i>n</i> where <i>n</i> is the depth
     * of <i>this</i> <code>Item</code> along the path returned by
     * <code>{@link #getPath()}</code>.
     *
     * @return The ancestor of the specified absolute degree of this
     * <code>Item</code> along the path returned by
     * <code>{@link #getPath()}</code>.
     *
     * @throws ItemNotFoundException if <i>degree</i> &lt; 0 or
     * <i>degree</i> &gt; <i>n</i> where <i>n</i> is the is the depth of
     * <i>this</i> <code>Item</code> along the path returned by
     * <code>{@link #getPath()}</code>.
     *
     * @throws AccessDeniedException if the current ticket does not have
     * sufficient access rights to complete the operation.
     *
     * @throws RepositoryException if another error occurs.
     */
    public Item getAncestor(int degree) throws ItemNotFoundException, AccessDeniedException, RepositoryException;

    /**
     * Returns the parent of this <code>Item</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * Returns the single parent (there is at most one) of this
     * <code>Item</code>.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * If this <code>Item</code> has multiple paths then this method returns
     * the parent along the path returned by <code>{@link #getPath()}</code>.
     *
     * @return The parent of this <code>Item</code> along the path returned
     * by <code>{@link #getPath()}</code>.
     *
     * @throws ItemNotFoundException if there is no parent.  This only happens
     * if <i>this</i> <code>Item</code> is the root node.
     *
     * @throws AccessDeniedException if the current ticket does not have
     * sufficient access rights to complete the operation.
     *
     * @throws RepositoryException if another error occurs.
     */
    public Node getParent() throws ItemNotFoundException, AccessDeniedException, RepositoryException;

//    /**
//     * Returns all parents of this <code>Item</code>.
//     * <p>
//     * <b>Level 1:</b>
//     * <p>
//     * Returns the (single) parent of this <code>Item</code>, wrapped in a
//     * <code>NodeIterator</code>.
//     * <p>
//     * <b>Level 2:</b>
//     * <p>
//     * Returns all the parents of this <code>Item</code>.
//     *
//     * @return An iterator over the parents of this <code>Item</code>.
//     *
//     * @throws ItemNotFoundException if there is no parent. This only happens
//     * if <i>this</i> <code>Item</code> is the root.
//     *
//     * @throws AccessDeniedException if the current ticket does not have
//     * sufficient access rights to complete the operation.
//     *
//     * @throws RepositoryException if another error occurs.
//     */
//    public NodeIterator getParents() throws ItemNotFoundException, AccessDeniedException, RepositoryException;

    /**
     * Returns the <code>Workspace</code> through which this <code>Item</code>
     * was acquired.
     * Every <code>Item</code> can ultimately be traced back through a series
     * of method calls to the call <code>{@link Workspace#getRootNode}</code> or
     * <code>{@link Workspace#getNodeByUuid}</code>. This method returns that
     * <code>Workspace</code> object.
     *
     * @return the <code>Workspace</code> through which this <code>Item</code> was
     * acquired.
     */
    public Workspace getWorkspace();

    /**
     * Accepts an <code>ItemVistor</code>.
     * Calls the appropriate <code>ItemVistor</code>
     * <code>visit</code> method of the according to whether <i>this</i>
     * <code>Item</code> is a <code>Node</code> or a <code>Property</code>.
     *
     * @param visitor The ItemVisitor to be accepted.
     *
     * @throws RepositoryException if an error occurs.
     */
    public void accept(ItemVisitor visitor) throws RepositoryException;

    /**
     * Indicates whether this <code>Item</code> is a <code>Node</code> or a
     * <code>Property</code>.
     * Returns <code>true</code> if this <code>Item</code> is a <code>Node</code>;
     * Returns <code>false</code> if this <code>Item</code> is a <code>Property</code>.
     *
     * @return <code>true</code> if this <code>Item</code> is a
     * <code>Node</code>, <code>false</code> if it is a <code>Property</code>.
     */
    public boolean isNode();

    /**
     * In level 2 returns the status of this <code>Item</code> regarding
     * the specified permissions, in level 1 always returns
     * <code>true</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * Always returns <code>true</code> since access control is not supported
     * in level 1 and therefore no access restrictions are in place.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * Returns the status of this <code>Item</code> regarding
     * the specified <code>permissions</code> (under the rights held by current
     * ticket). See <code>{@link javax.jcr.access.Permission}</code> and
     * <code>{@link javax.jcr.access.AccessManager}</code> for more details. Equivalent to
     * <code>AccessManager.isGranted(<b>this</b>, permissions)</code>.
     *
     * @param permissions A combination of one or more of the following constants
     * encoded as a bitmask value:
     * <ul>
     * <li><code>Permission.ADD_NODE</code></li>
     * <li><code>Permission.ADD_PROPERTY</code></li>
     * <li><code>Permission.REMOVE_ITEM</code></li>
     * <li><code>Permission.READ_ITEM</code></li>
     * <li><code>Permission.WRITE_PROPERTY</code></li>
     * </ul>
     * See <code>{@link javax.jcr.access.Permission}</code>.
     *
     * @return In level 1: Always <code>true</code>. In level 2:
     * <code>true</code> if all requested permissions are true for this
     * <code>Item</code>; <code>false</code> otherwise.
     *
     * @throws IllegalArgumentException if <code>permissions</code> specifies
     * invalid permissions.
     */
    public boolean isGranted(long permissions);

    /**
     * Returns the depth of this <code>Item</code> in the repository tree.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * Returns the depth below the root node of <i>this</i> <code>Item</code>
     * (counting <i>this</i> <code>Item</code> itself).
     * <ul>
     * <li>The root node returns 0.
     * <li>A property or child node of the root node returns 1.
     * <li>A property or child node of a child node of the root returns 2.
     * <li>And so on to <i>this</i> <code>Item</code>.
     * </ul>
     * <p>
     * <b>Level 2:</b>
     * <p>
     * Same behaviour as level 1, but if multiple paths exist to this
     * <code>Item</code> then the path used to determine the depth is the
     * same one that is returned by <code>{@link #getPath()}</code>.
     *
     * @return The depth of this <code>Item</code> in the repository hierarchy.
     */
    public int getDepth();

    /**
     * In level 2 places a lock on this node, in level 1 throws an
     * <code>UnsupportedRepositoryOperationException</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * Throws an <code>UnsupportedRepositoryOperationException</code> since
     * locking is not supported in level 1.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * Puts a lock on this item.
     * <p>
     * If <code>recurse</code> is <code>true</code> then a lock is also placed
     * on all descendants of this item. If <code>false</code> then only this
     * item is locked.
     * <p>
     * If <code>shared</code> is <code>true</code> then this lock is
     * a shared lock. If <code>false</code> then it is an exclusive lock.
     * <p>
     * The lock is of the specified <code>lockType</code>. This standard
     * defines only one type, <code>LockType.WRITE_LOCK</code>, though more can
     * be added by the implementation, see <code>{@link javax.jcr.lock.LockType LockType}</code> for the
     * semantics specific to this lock type.
     * <p>
     * <b>General points about locking</b>
     * <p>
     * Locking is an optional part of the specification. If locking is not
     * supported then this method will throw an
     * <code>UnsupportedRepositoryOperationException</code>, even in a level 2
     * implementation.
     * <p>
     * Even if locking is supported, the features of a particular
     * implementation can vary along the following dimensions:
     * <dl>
     * <dt>Lock Type</dt>
     * <dd>Only WRITE_LOCK is defined by this standard. An implementation may
     * define other lock types.</dd>
     * <dt>Lock Scope</dt>
     * <dd>An implementation may choose to implement exclusive locks, shared
     * locks or both.</dd>
     * <dt>Lock Level</dt>
     * <dd>An implementation may choose to implement locking on
     * nodes only or on both nodes and properties. If the implementation locks
     * only on the node level this is equivalent to a
     * "both nodes and properties" locking implementaion in which locking or
     * unlocking a node automatically performs the same operation on all its
     * properties.</dd>
     * </dl>
     * Information about what locking support is provided is available via
     * the method <code>{@link Workspace#getLockCapabilities()}</code> which returns
     * a <code>{@link javax.jcr.lock.LockCapabilities LockCapabilities}</code> object which provides methods for
     * querying the locking support provided by the implementation.
     *
     * @param recurse if <code>true</code> set locks on all descendants also.
     *
     * @param shared Whether the lock is shared or exclusive
     *
     * @param lockType The type of the lock
     *
     * @return The Lock object.
     *
     * @throws AccessDeniedException In level 1: Never. In level 2 if the user
     * ticket does not have the right to lock this user <i>or</i> does have
     * the right to lock this node but an exclusive lock is already held by
     * another user.
     *
     * @throws UnsupportedRepositoryOperationException In level 1: Always.
     * In level 2: If the lock type or <code>shared</code> status is not supported in this
     * implementation or locking in general is not supported.
     *
     * @throws IllegalArgumentException if <code>lockType</code> is not a valid lock type.
     * @throws RepositoryException if another error occurs.
     */
    public Lock lock(boolean recurse, boolean shared, int lockType) throws UnsupportedRepositoryOperationException, AccessDeniedException, RepositoryException;

    /**
     * In level 2 places a lock on this node, in level 1 throws an
     * <code>UnsupportedRepositoryOperationException</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * Throws an <code>UnsupportedRepositoryOperationException</code>.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * Sets a write lock on this node and descendants. If
     * <code>recurse</code> is <code>true</code> then this node and all its
     * descendants are locked. Otherwise only this node is locked.
     * Equivalent to <code>lock(recurse, shared, LockType.WRITE_LOCK)</code>.
     *
     * @param shared Whether the lock is shared or exclusive.
     *
     * @param recurse if <code>true</code> set locks on all descendants also.
     *
     * @return The Lock object.
     *
     * @throws AccessDeniedException In level 1: Never. In level 2 if the user
     * ticket does not have the right to lock this user <i>or</i> does have
     * the right to lock this node but an exclusive lock is already held by
     * another user.
     *
     * @throws UnsupportedRepositoryOperationException In level 1: Always.
     * In level 2: If the <code>shared</code> status is not
     * supported in this implementation or locking in general is not supported.
     * 
     * @throws RepositoryException if another error occurs.
     */
    public Lock lock(boolean recurse, boolean shared) throws UnsupportedRepositoryOperationException, AccessDeniedException, RepositoryException;

    /**
     * In level 2 places a lock on this node, in level 1 throws an
     * <code>UnsupportedRepositoryOperationException</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * Throws an <code>UnsupportedRepositoryOperationException</code>.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * Sets an exclusive (not shared) write lock on this node. If
     * <code>recurse</code> is <code>true</code> then this node and all its
     * descendants are locked. Otherwise only this node is locked. Equivalent to
     * <code>lock(recurse, false, LockType.WRITE_LOCK)</code>.
     *
     * @param recurse if <code>true</code> set locks on all descendants also.
     *
     * @return The Lock object.
     *
     * @throws AccessDeniedException In level 1: Never. In level 2 if the user
     * ticket does not have the right to lock this user <i>or</i> does have
     * the right to lock this node but an exclusive lock is already held by
     * another user.
     *
     * @throws UnsupportedRepositoryOperationException In level 1: Always.
     * In level 2: If locking in general is not supported.
     * 
     * @throws RepositoryException if another error occurs.
     */
    public Lock lock(boolean recurse) throws UnsupportedRepositoryOperationException, AccessDeniedException, RepositoryException;

    /**
     * In level 2 places a lock on this node, in level 1 throws an
     * <code>UnsupportedRepositoryOperationException</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * Throws an <code>UnsupportedRepositoryOperationException</code>.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * Sets an exclusive (not shared) write lock on this node (but not its
     * descendants). Equivalent to
     * <code>lock(false, false, LockType.WRITE_LOCK)</code>.
     *
     * @return The Lock object.
     *
     * @throws AccessDeniedException In level 1: Never. In level 2 if the user
     * ticket does not have the right to lock this user <i>or</i> does have
     * the right to lock this node but an exclusive lock is already held by
     * another user.
     *
     * @throws UnsupportedRepositoryOperationException In level 1: Always.
     * In level 2: If locking in general is not supported.
     * 
     * @throws RepositoryException if another error occurs.
     */
    public Lock lock() throws UnsupportedRepositoryOperationException, AccessDeniedException, RepositoryException;

    /**
     * In level 2 removes the specified lock from this node, in level 1 throws an
     * <code>UnsupportedRepositoryOperationException</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * Throws an <code>UnsupportedRepositoryOperationException</code>.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * If this node is not locked, then this method has no effect.
     * If this node is locked and the user of this ticket is a user of the
     * lock then the lock is removed.  If this node is locked but the user of
     * this ticket does not have the power to unlock it, then an
     * <code>AccessDeniedException</code> is thrown.
     *
     * @throws AccessDeniedException In level 1: Never. In level 2: If the user
     * of this ticket does not have the right to unlock this node.
     *
     * @throws UnsupportedRepositoryOperationException In level 1: Always.
     * In level 2: If locking in general is not supported.
     * 
     * @throws RepositoryException if another error occurs.
     */
    public void unlock(Lock lock) throws UnsupportedRepositoryOperationException, AccessDeniedException, RepositoryException;

    /**
     * In level 2 gets all locks on this node, in level 1 throws an
     * <code>UnsupportedRepositoryOperationException</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * Throws an <code>UnsupportedRepositoryOperationException</code>.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * Returns all locks on this node, if there are any, otherwise returns
     * <code>null</code>.
     *
     * @return The locks on this node
     *
     * @throws UnsupportedRepositoryOperationException In level 1: Always.
     * In level 2: if locking in general is not supported.
     * @throws RepositoryException if another error occurs.
     */
    public Lock[] getLocks() throws UnsupportedRepositoryOperationException, RepositoryException;

    /**
     * Indicates whether this item has any locks on it.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * Always returns <code>false</code>.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * Returns <code>true</code> if there is a lock on this node, false
     * otherwise.
     *
     * @return In level 1: Always <code>false</code>. In level 2:
     * <code>true</code> if there is a lock on this node, false
     * otherwise.
     *
     * @throws RepositoryException if an error occurs.
     */
    public boolean hasLocks() throws RepositoryException;

    /**
     * Returns <code>true</code> if <code>this</code> <code>Item</code>
     * (the Java object) represents the same repository item as the Java
     * object <code>otherItem</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * In some implementations the results of <code>Item.isIdentical</code>
     * and <code>Object.equals</code> will not always be the same. Therefore,
     * in general, clients of the API cannot rely on <code>Object.equals</code>
     * to test for identity of repository items and must use this method.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * Same as level 1 except that <i>all</i> level 2 implementations will
     * be such that results of <code>Item.isIdentical</code>
     * and <code>Object.equals</code> will not always be the same.
     * This is a result of hard links: A particular instance (Java object)
     * of a hard-linked node returns a different path from
     * <code>getPath()</code> depending on the path through which it was acquired.
     * This means that it must be possible for two <code>Node</code> objects
     * with different internal state to represent the same actual node.
     * Hence, the necessity of using this method for testing item identity.
     *
     * @param otherItem the item to be tested for identity with
     * <code>this</code>.
     *
     * @return a <code>boolean</code>.
     */
    public boolean isIdentical(Item otherItem);
}