/*
 * Node.java
 *
 * This file is part of the specification for the
 * Java Content Repository API.
 */
package javax.jcr;

import java.io.InputStream;
import java.util.Calendar;

import javax.jcr.access.AccessDeniedException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.nodetype.NodeDef;
import javax.jcr.nodetype.NodeType;
import javax.jcr.version.InvalidVersionException;
import javax.jcr.version.Version;
import javax.jcr.version.VersionException;

/**
 * The <code>Node</code> interface represents a node in the hierarchy that
 * makes up the repository. Many methods of this interface take a
 * <code>path</code> parameter. If the path starts with a <code>/</code>
 * character then it is interpreted as an absolute path, starting at the root
 * node of the repository. If it does not begin with a <code>/</code>, then it
 * is interpreted as a relative path, relative to <i>this</i> node.
 * <p>
 * For example, if a node <code>b</code> has absolute path <code>/a/b</code>
 * then:
 * <ul>
 * <li>
 * <code>b.addNode("c")</code> will add a new node at <code>/a/b/c</code>.
 * The node <code>c</code> is created as a direct child of <code>b</code>.
 * <li>
 * <code>b.addNode("c/d")</code> will add a new node at <code>/a/b/c/d</code>
 * (assuming a node at <code>/a/b/c</code> already exists).
 * <li>
 * <code>b.addNode("/x/y/z")</code> will add a new node at <code>/x/y/z</code>
 * (assuming a node at <code>/x/y</code> already exists. Notice that in this
 * case the fact that we called the method from <code>b</code> is irrelevant).
 * </ul>
 * Both nodes and properties use the same path syntax. For example,
 * b.addProperty("c/d/e") adds property <code>e</code> to node
 * <code>/a/b/c/d</code> (assuming <code>/a/b/c/d</code> already exists).
 * <p>
 * The last delimited item in the path (for example, <code>c</code> in
 * <code>/a/b/c</code>) is the <i>name</i> of the node or property.
 * <p>
 * The unix-style ".." (parent) and "." (this) are not supported.
 * <p>
 *
 * @author Peeter Piegaze
 * @author Stefan Guggisberg
 */
public interface Node extends Item {

    /**
     * Creates a new node at <code>path</code>.
     * The new node will only be persisted in the workspace when
     * <code>save()</code> is called on the new node's parent node or
     * other ancestor node (parent's parent, etc.) and if the structure
     * of the new node (its child nodes and properties) meets the
     * constraint criteria of the parent node's node type.
     * <p>
     * If <code>path</code> implies intermediary nodes that do not
     * exist then a <code>NoSuchPathException</code> is thrown.
     * <p>
     * If an item already exists at <code>path</code> then an
     * <code>ItemExistsException</code> is thrown.
     * <p>
     * If an attempt is made to add a node as a child of a
     * property then a <code>RepositoryException</code> is
     * thrown.
     * <p>
     * The new node's node type will be determined by the node type
     * of its parent.
     *
     * @param path The path of the new node to be created.
     *
     * @return The node that was added.
     *
     * @throws ItemExistsException If an item at the
     * specified path already exists.
     *
     * @throws PathNotFoundException If the specified path implies intermediary
     * nodes that do not exist.
     *
     * @throws RepositoryException If an attempt
     * is made to add a node as the child of a
     * <code>Property</code> or if another error occurs.
     */
    public Node addNode(String path) throws ItemExistsException, PathNotFoundException, RepositoryException;

    /**
     * Creates a new node at <code>path</code> of the specified node type.
     * The same as <code>addNode(String path)</code> except that the type of the
     * new node is explictly specified.
     * <p>
     * If <code>nodeTypeName</code> is not
     * recognized, then a <code>NoSuchNodeTypeException</code> is thrown.
     *
     * @param path The path of the new <code>Node</code> that is to be created,
     * the last item of this pathwill be the name of the new <code>Node</code>.
     *
     * @param nodeTypeName The name of the node type of the new node.
     *
     * @return The node that was added.
     *
     * @throws ItemExistsException If an item at the
     * specified path already exists.
     *
     * @throws PathNotFoundException If specified path implies intermediary
     * <code>Node</code>s that do not exist.
     *
     * @throws NoSuchNodeTypeException If the specified <code>nodeTypeName</code>
     * is not recognized.
     *
     * @throws RepositoryException If an attempt
     * is made to add a node as the child of a
     * <code>Property</code> or if another error occurs.
     */
    public Node addNode(String path, String nodeTypeName) throws ItemExistsException, PathNotFoundException, NoSuchNodeTypeException, RepositoryException;

//    /**
//     * In level 2, adds an existing child to this node.
//     * <p>
//     * <b>Level 1:</b>
//     * <p>
//     * Throws <code>UnsupportedRepositoryOperationException</code>.
//     * <p>
//     * <b>Level 2:</b>
//     * <p>
//     * Link the existing node at <code>absPath</code> as a new child of this
//     * node. The new child has the same name in the new path (the one where this
//     * node is its parent) as it does in <code>absPath</code>.
//     *
//     * @param absPath The absolute path of the new child node.
//     *
//     * @return The new child node.
//     *
//     * @throws PathNotFoundException In level 2: If no node exists at <code>absPath</code>.
//     *
//     * @throws UnsupportedRepositoryOperationException In level 1: Always.
//     *
//     * @throws RepositoryException In level 2: If another error occurs.
//     * 
//     * @deprecated will be removed in a few builds
//     */
//    public Node addExistingNode(String absPath) throws UnsupportedRepositoryOperationException, PathNotFoundException, RepositoryException;

//    /**
//     * In level 2, adds an existing child to this node.
//     * <p>
//     * <b>Level 1:</b>
//     * <p>
//     * Throws <code>UnsupportedRepositoryOperationException</code>.
//     * <p>
//     * <b>Level 2:</b>
//     * <p>
//     * Same as <code>{@link #addExistingNode(String absPath)}</code> except that
//     * the node at <code>absPath</code> adopts <code>newName</code> in the path
//     * where this node is its parent.
//     *
//     * @param absPath The absolute path of the new child node.
//     *
//     * @param newName The new name for this node when referenced as a child of this node.
//     *
//     * @return The new child node.
//     *
//     * @throws PathNotFoundException In level 2: If no node exists at <code>absPath</code>.
//     *
//     * @throws UnsupportedRepositoryOperationException In level 1: Always.
//     *
//     * @throws RepositoryException In level 2: If another error occurs.
//     * 
//     * @deprecated will be removed in a few builds
//     */
//    public Node addExistingNode(String absPath, String newName) throws UnsupportedRepositoryOperationException, PathNotFoundException, RepositoryException;

    /**
     * Sets the specified property to the specified value. If the property does
     * not yet exist, it is created. The property type of the property will be
     * that specified by the node type of <code>this</code> node (the one on
     * which this method is being called). If the </code>PropertyType</code>
     * of the supplied <code>Value</code> object (recall that a
     * <code>{@link Value}</code> object records the property type of its
     * contained value) is different from that required, a best-effort
     * conversion is attempted.
     * <p>
     * If the node type of the parent node does not specify a
     * specific property type for the property being set, then
     * the property type of the supplied <code>Value</code> object
     * is used.
     * <p>
     * If the property already exists (has previously been set) it assumes both the new value
     * and new type (if different). To erase a property, use
     * <code>remove(String path)</code>.
     * <p>
     * To persist the addition or change of a property to the workspace
     * <code>{@link #save}</code> must be called on
     * this node (the parent of the property being set) or a higher-order
     * ancestor of the property.
     *
     * @param name The name of a property of this node
     *
     * @param value The value to be assigned
     *
     * @return The updated <code>Property</code> object
     *
     * @throws StaleValueException If (in implementations that support this)
     * the value has changed via another Ticket, since it was last retrieved.
     *
     * @throws RepositoryException If another error occurs.
     */
    public Property setProperty(String name, Value value) throws StaleValueException, RepositoryException;
    
    /**
     * Sets the specified property to the specified values. If the property does
     * not yet exist, it is created. The property type of the property will be
     * that specified by the node type of <code>this</code> node (the one on
     * which this method is being called). If the </code>PropertyType</code>
     * of the supplied <code>Value</code> object (recall that a
     * <code>{@link Value}</code> object records the property type of its
     * contained value) is different from that required, a best-effort
     * conversion is attempted.
     * <p>
     * If the node type of the parent node does not specify a
     * specific property type for the property being set, then
     * the property type of the supplied <code>Value</code> object
     * is used.
     * <p>
     * If the property already exists (has previously been set) it assumes both the new value
     * and new type (if different). To erase a property, use
     * <code>remove(String path)</code>.
     * <p>
     * To persist the addition or change of a property to the workspace
     * <code>{@link #save}</code> must be called on
     * this node (the parent of the property being set) or a higher-order
     * ancestor of the property.
     *
     * @param name The name of a property of this node
     *
     * @param values The values to be assigned
     *
     * @return The updated <code>Property</code> object
     *
     * @throws StaleValueException If (in implementations that support this)
     * the value has changed via another Ticket, since it was last retrieved.
     *
     * @throws RepositoryException If another error occurs.
     */
    public Property setProperty(String name, Value[] values) throws StaleValueException, RepositoryException;

    /**
     * Sets the specified property to the specified value and the specified type.
     * If the property does not yet exist, it is created.
     * <p>
     * If the node type of the parent node does not specify a
     * specific property type for the property being set, then
     * the property type specified by the <code>type</code>
     * parameter is used.
     * <p>
     * If the property already exists (has previously been set) it assumes both
     * the new value and new type (if different). To erase a property, use
     * <code>remove(String path)</code>.
     * <p>
     * To persist the addition or change of a property to the workspace
     * <code>save()</code> must be called on
     * <code>this</code> node (the parent of the property being set) or a higher-order
     * ancestor of the property.
     *
     * @param name The name of a property of this node
     *
     * @param value The value to be assigned
     *
     * @param type The type of the property
     *
     * @return The updated <code>Property</code> object
     *
     * @throws ValueFormatException if the type or format of <code>value</code>
     * is incompatible with the type of the specified property or if
     * <code>value</code> is incompatible with (i.e. can not be converted to)
     * <code>type</code>.
     *
     * @throws StaleValueException If (in implementations that support this)
     * the value has changed via another Ticket, since it was last retrieved.
     *
     * @throws RepositoryException If another error occurs.
     */
    public Property setProperty(String name, Value value, int type) throws ValueFormatException, StaleValueException, RepositoryException;
    
    /**
     * Sets the specified property to the specified values and the specified type.
     * If the property does not yet exist, it is created.
     * <p>
     * If the node type of the parent node does not specify a
     * specific property type for the property being set, then
     * the property type specified by the <code>type</code>
     * parameter is used.
     * <p>
     * If the property already exists (has previously been set) it assumes both
     * the new values and new type (if different). To erase a property, use
     * <code>remove(String path)</code>.
     * <p>
     * To persist the addition or change of a property to the workspace
     * <code>save()</code> must be called on
     * <code>this</code> node (the parent of the property being set) or a higher-order
     * ancestor of the property.
     *
     * @param name The name of a property of this node
     *
     * @param values The values to be assigned
     *
     * @param type The type of the property
     *
     * @return The updated <code>Property</code> object
     *
     * @throws ValueFormatException if the type or format of <code>values</code>
     * is incompatible with the type of the specified property or if
     * <code>values</code> is incompatible with (i.e. can not be converted to)
     * <code>type</code>.
     *
     * @throws StaleValueException If (in implementations that support this)
     * the value has changed via another Ticket, since it was last retrieved.
     *
     * @throws RepositoryException If another error occurs.
     */
    public Property setProperty(String name, Value[] values, int type) throws ValueFormatException, StaleValueException, RepositoryException;
    

    /**
     * Sets the specified property to the specified value and the specified type.
     * If the property does not yet exist, it is created.
     * Same as {@link #setProperty(String, Value, int)
     * setProperty(String name, Value value, int type)} except the value is
     * passed as a <code>String</code> instead of a <code>Value</code> object.
     *
     * @param name The name of a property of this node
     *
     * @param value The value to assigned
     *
     * @param type The type of the property
     *
     * @return The updated <code>Property</code> object
     *
     * @throws ValueFormatException if the type or format of <code>value</code>
     * is incompatible with the type of the specified property or if
     * <code>value</code> is incompatible with (i.e. can not be converted to)
     * <code>type</code>.
     *
     * @throws StaleValueException If (in implementations that support this)
     * the value has changed via another Ticket, since it was last retrieved.
     *
     * @throws RepositoryException If another error occurs.
     */
    public Property setProperty(String name, String value, int type) throws ValueFormatException, StaleValueException, RepositoryException;
    
    /**
     * Sets the specified property to the specified values and the specified type.
     * If the property does not yet exist, it is created.
     * Same as {@link #setProperty(String, Value[], int)
     * setProperty(String name, Value[] values, int type)} except the value is
     * passed as a <code>String</code>[] instead of a <code>Value</code>[] object.
     *
     * @param name The name of a property of this node
     *
     * @param values The value to assigned
     *
     * @param type The type of the property
     *
     * @return The updated <code>Property</code> object
     *
     * @throws ValueFormatException if the type or format of <code>values</code>
     * is incompatible with the type of the specified property or if
     * <code>values</code> is incompatible with (i.e. can not be converted to)
     * <code>type</code>.
     *
     * @throws StaleValueException If (in implementations that support this)
     * the value has changed via another Ticket, since it was last retrieved.
     *
     * @throws RepositoryException If another error occurs.
     */
    public Property setProperty(String name, String[] values, int type) throws ValueFormatException, StaleValueException, RepositoryException;    

    /**
     * Sets the specified property to the specified value.
     * If the property does not yet exist, it is created.
     * The property type of the property being set is determined
     * by the node type of <code>this</code> node (the one on which
     * this method is being called). If this is something other than
     * <code>PropertyType.BINARY</code>, a best-effort conversion is attempted.
     * If the conversion fails, a <code>ValueFormatException</code> is
     * thrown.
     * <p>
     * If the node type of <code>this</code> node does not
     * specify a particular property type for the property being set
     * then <code>PropertyType.BINARY</code> is used.
     * <p>
     * If the property
     * already exists (has previously been set) it assumes both the new value
     * and new type (if different). To erase a property, use
     * <code>remove(String path)</code>.
     * <p>
     * To persist the addition or change of a property to the workspace
     * <code>save()</code> must be called on
     * this node (the parent of the property being set) or a higher-order
     * ancestor of the property.
     *
     * <i>NOTE: To create a property of PropertyType.SOFTLINK an explicit type
     * must be specified using a three-argument signature.</i>
     *
     * @param name The name of a property of this node
     *
     * @param value The value to assigned
     *
     * @return The updated <code>Property</code> object
     *
     * @throws ValueFormatException if <code>value</code> is incompatible with
     * (i.e. can not be converted to) the type of the specified property.
     *
     * @throws StaleValueException If (in implementations that support this)
     * the value has changed via another Ticket, since it was last retrieved.
     *
     * @throws RepositoryException If another error occurs.
     */
    public Property setProperty(String name, InputStream value) throws ValueFormatException, StaleValueException, RepositoryException;

    /**
     * Sets the specified property to the specified value.
     * If the property does not yet exist, it is created.
     * Same as <code>{@link #setProperty(String name, InputStream value)}</code>
     * except that the value passed is a <code>boolean</code> and therefore the implied
     * property type is <code>PropertyType.BOOLEAN</code>.
     *
     * @param name The name of a property of this node
     *
     * @param value The value to assigned
     *
     * @return The updated <code>Property</code> object
     *
     * @throws ValueFormatException if <code>value</code> is incompatible with
     * (i.e. can not be converted to) the type of the specified property.
     *
     * @throws StaleValueException If (in implementations that support this)
     * the value has changed via another Ticket, since it was last retrieved.
     *
     * @throws RepositoryException If another error occurs.
     */
    public Property setProperty(String name, boolean value) throws ValueFormatException,  StaleValueException, RepositoryException;

    /**
     * Sets the specified property to the specified value.
     * If the property does not yet exist, it is created.
     * Same as <code>{@link #setProperty(String name, InputStream value)}</code>
     * except that the value passed is a <code>double</code> and therefore the implied
     * property type is <code>PropertyType.DOUBLE</code>.
     *
     * @param name The name of a property of this node
     *
     * @param value The value to assigned
     *
     * @return The updated <code>Property</code> object
     *
     * @throws ValueFormatException if <code>value</code> is incompatible with
     * (i.e. can not be converted to) the type of the specified property.
     *
     * @throws StaleValueException If (in implementations that support this)
     * the value has changed via another Ticket, since it was last retrieved.
     *
     * @throws RepositoryException If another error occurs.
     */
    public Property setProperty(String name, double value) throws ValueFormatException, StaleValueException, RepositoryException;

    /**
     * Sets the specified property to the specified value.
     * If the property does not yet exist, it is created.
     * Same as <code>{@link #setProperty(String name, InputStream value)}</code>
     * except that the value passed is a <code>long</code> and therefore the implied
     * property type is <code>PropertyType.LONG</code>.
     *
     * @param name The name of a property of this node
     *
     * @param value The value to assigned
     *
     * @return The updated <code>Property</code> object
     *
     * @throws ValueFormatException if <code>value</code> is incompatible with
     * (i.e. can not be converted to) the type of the specified property.
     *
     * @throws StaleValueException If (in implementations that support this)
     * the value has changed via another Ticket, since it was last retrieved.
     *
     * @throws RepositoryException If another error occurs.
     */
    public Property setProperty(String name, long value) throws ValueFormatException, StaleValueException, RepositoryException;

    /**
     * Sets the specified property to the specified value.
     * If the property does not yet exist, it is created.
     * Same as <code>{@link #setProperty(String name, InputStream value)}</code>
     * except that the value passed is a <code>Calendar</code> and therefore the implied
     * property type is <code>PropertyType.DATE</code>.
     *
     * @param name The name of a property of this node
     *
     * @param value The value to assigned
     *
     * @return The updated <code>Property</code> object
     *
     * @throws ValueFormatException if <code>value</code> is incompatible with
     * (i.e. can not be converted to) the type of the specified property.
     *
     * @throws StaleValueException If (in implementations that support this)
     * the value has changed via another Ticket, since it was last retrieved.
     *
     * @throws RepositoryException If another error occurs.
     */
    public Property setProperty(String name, Calendar value) throws ValueFormatException, StaleValueException, RepositoryException;
    
  /**
   * Sets the specified property to the specified value.
   * If the property does not yet exist, it is created.
   * Same as <code>{@link #setProperty(String name, InputStream value)}</code>
   * except that the value passed is a <code>String</code> and therefore the implied
   * property type is <code>PropertyType.String</code>.
   *
   * @param name The name of a property of this node
   *
   * @param value The value to assigned
   *
   * @return The updated <code>Property</code> object
   *
   * @throws ValueFormatException if <code>value</code> is incompatible with
   * (i.e. can not be converted to) the type of the specified property.
   *
   * @throws StaleValueException If (in implementations that support this)
   * the value has changed via another Ticket, since it was last retrieved.
   *
   * @throws RepositoryException If another error occurs.
   */
  public Property setProperty(String name, String value) throws ValueFormatException, StaleValueException, RepositoryException;

    /**
     * Removes the item at <code>path</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * Removes (that is, actually erases) the item at <code>path</code> and
     * all of that item's descendants.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * Same as level 1 except, if the item has more than one parent (through
     * hard linking) then only the <i>reference</i> from the particular parent
     * specified by <code>path</code> to this node is removed. The item will
     * no longer be found at <code>path</code>, though it may still
     * be accessible via other paths.
     *
     * @throws PathNotFoundException if no <code>Item</code> exists at
     *         <code>path</code>.
     *
     * @throws RepositoryException if another error occurs.
     */
    public void remove(String path) throws PathNotFoundException, RepositoryException;

    /**
     * Persists all changes to this Node and its descendants made since the
     * last save or get of this Node.
     * Applies to all changes made to all descendants of this node (since last
     * save or get) including child nodes added and properties set.
     * Constraints mandated by node types are
     * validated on save. If validation fails all changes made since the last
     * save or get are rolled-back and a ConstraintViolation exception is
     * thrown. The save method allows a Node to be in a temporarily invalid
     * state which it is being "built", that is, while it is having properties
     * and child nodes added to it. Once it is "finished", <code>save</code>
     * is called and the structure is validated.
     *
     * @throws ConstraintViolationException If any of the changes would
     * violate a constraint as defined by the node type of the respective node.
     *
     * @throws AccessDeniedException if the current ticket does not have
     * sufficient access rights to complete the operation.
     *
     * @throws RepositoryException If another error occurs.
     */
    public void save() throws AccessDeniedException, ConstraintViolationException, RepositoryException;

    /**
     * Reverts this Node and its descendants to the state they were in before
     * the changes persisted in the last save.
     * Applies to all changes made to all descendants of this node (since last
     * save or get) including child nodes added and properties set.
     *
     * @throws RepositoryException If an unexpected error occurs.
     */
    public void revert() throws RepositoryException;

    /**
     * Updates this node to reflect the state (i.e., have the same properties and child nodes) of
     * this node's corresponding node in srcWorkspace (that is, the node in srcWorkspace with the
     * same UUID as this node). If shallow is set to false, then only this node is updated. If
     * shallow is set to true then every node with a UUID in the subtree rooted at this node is
     * updated.  If the update succeeds the changes made to this node are persisted immediately,
     * there is no need to call save.
     *
     * @param srcWorkspace The name of the source workspace used to update this node.
     * @param shallow If true, only the current node is updated; otherwise, the subtree with this node
     * as its root is updated.
     *
     * @throws AccessDeniedException if the current ticket does not have
     * sufficient access rights to complete the operation.
     *
     * @throws RepositoryException if another error occurs.
     */
    public void update(String srcWorkspace, boolean shallow) throws AccessDeniedException, ItemNotFoundException, RepositoryException;

    /**
     * Performs the same function as update with one restriction: merge only succeeds if the base
     * version of the corresponding node in srcWorkspace is a successor (or a successor of a successor,
     * etc., to any degree of separation) of the base version of this node. Otherwise, the operation
     * throws a MergeException.  If the merge succeeds the changes made to this node are persisted
     * immediately, there is no need to call save.  If versioning is not supported, merge is equivalent
     * to update.
     *
     * @param srcWorkspace The name of the source workspace used to merge this node.
     * @param shallow If true, only the current node is merged; otherwise, the subtree with this node
     * as its root is merged.
     *
     * @throws AccessDeniedException if the current ticket does not have
     * sufficient access rights to complete the operation.
     *
     * @throws InvalidVersionException if the version of the node in srcWorkspace is not a successor of
     * this node (or a descendant node if shallow is false).
     *
     * @throws ItemNotFoundException if a corresponding node (with same UUID) does not exist in srcWorkspace.
     *
     * @throws RepositoryException if another error occurs.
     */
    public void merge(String srcWorkspace, boolean shallow) throws AccessDeniedException, InvalidVersionException, ItemNotFoundException, RepositoryException;

    /**
     * Get the node at <code>path</code>.
     * Returns the specified node.
     *
     * @param path The path of the node to retrieve.
     *
     * @return The node at <code>path</code>.
     *
     * @throws PathNotFoundException If no node exists at the
     * specified path.
     *
     * @throws RepositoryException If another error occurs.
     */
    public Node getNode(String path) throws PathNotFoundException,  RepositoryException;

    /**
     * Returns all child nodes of this node.
     * Returns a <code>NodeIterator</code> over all child <code>Node</code>s of
     * this <code>Node</code>. Does <i>not</i> include properties of this
     * <code>Node</code>.
     *
     * @return A <code>NodeIterator</code> over all child <code>Node</code>s of
     *         this <code>Node</code>.
     *
     * @throws RepositoryException If an unexpected error occurs.
     */
    public NodeIterator getNodes() throws RepositoryException;


    /**
     * Gets all child nodes of this node that match <code>namePattern</code>.
     * The pattern may be a full name or a partial name with one or more
     * wildcard characters ("<code>*</code>"), or a disjunction (using the
     * "<code>|</code>" character to represent logical <code>OR</code>) of
     * these. For example,
     * <p>
     * <code>N.getNodes("jcr:* | myapp:report")</code>
     * <p>
     * would return a <code>NodeIterator</code> holding all child nodes of
     * <code>N</code> that are either called <code>myapp:report</code> or begin
     * with the prefix <code>jcr:</code>. The pattern does not represent paths,
     * that is, it may not contain <code>/</code> characters.
     *
     * @param namePattern a name pattern
     * @return a <code>NodeIterator</code>
     *
     * @throws RepositoryException If an unexpected error occurs.
     */
    public NodeIterator getNodes(String namePattern) throws RepositoryException;

    /**
     * Get the property at <code>path</code>.
     * Returns the specified property.
     *
     * @param path The path of the property to retrieve.
     *
     * @return The property at <code>path</code>.
     *
     * @throws PathNotFoundException If no property exists at the
     * specified path.
     *
     * @throws RepositoryException If another error occurs.
     */
    public Property getProperty(String path) throws PathNotFoundException, RepositoryException;

    /**
     * Returns all properties of this node.
     * Returns a <code>PropertyIterator</code> over all properties
     * of this node. Does <i>not</i> include child <i>nodes</i> of this
     * node.
     *
     * @return A <code>PropertyIterator</code>.
     *
     * @throws RepositoryException If an unexpected error occurs.
     */
    public PropertyIterator getProperties() throws RepositoryException;

    /**
     * Gets all properties of this node that match <code>namePattern</code>.
     * The pattern may be a full name or a partial name with one or more
     * wildcard characters ("<code>*</code>"), or a disjunction (using the
     * "<code>|</code>" character to represent logical <code>OR</code>) of
     * these. For example,
     * <p>
     * <code>N.getProperties("jcr:* | myapp:name")</code>
     * <p>
     * would return a <code>PropertyIterator</code> holding all properties of
     * <code>N</code> that are either called <code>myapp:name</code> or begin
     * with the prefix <code>jcr:</code>. The pattern does not represent paths,
     * that is, it may not contain <code>/</code> characters.
     *
     * @param namePattern a name pattern
     * @return a <code>PropertyIterator</code>
     *
     * @throws RepositoryException If an unexpected error occurs.
     */
    public PropertyIterator getProperties(String namePattern) throws RepositoryException;

    /**
     * Returns all properties of <i>this</i> node with the specified value.
     * If the spedified value and the value of a property are of different types
     * then a conversion is attempted before the equality test is made. Returns
     * an empty iterator if no such property could be found. In the case of
     * multivalue properties, a property qualifies as having the specified value
     * if and only if at least one of its values matches.
     *
     * @param value A <code>Value</code> object.
     *
     * @return  A PropertyIterator holding all properties of this node with the
     * specified value. Returns an empty iterator if no such property could be found.
     *
     * @throws RepositoryException If an unexpected error occurs.
     */
    public PropertyIterator findProperties(Value value) throws RepositoryException;

    /**
     * Returns the first property of <i>this</i> node found with the specified value.
     * What makes a particular property "first" (that is, the search order) is
     * implementaion dependent. If the specified value and the value of a
     * property are of different types then a conversion is attempted before the
     * equality test is made. Returns <code>null</code> if no such property is
     * found. In the case of multivalue properties, a property qualifies as
     * having the specified value if and only if at least one of its values matches.
     *
     * @param value A <code>Value</code> object.
     *
     * @return  The first property of <i>this</i> node found with the specified value.
     *
     * @throws RepositoryException If an unexpected error occurs.
     */
    public Property findProperty(Value value) throws RepositoryException;

    /**
     * Returns all child nodes and properties of this node that pass the filter.
     * Returns an <code>ItemIterator</code>, preserving order if it exists.
     * The specified <code>filter</code> must implement the interface
     * <code>{@link ItemFilter}</code> (which has a single method, <code>accept</code>).
     * The returned <code>ItemIterator</code> includes all child items
     * <code>e</code> of <code>this</code> node for which
     * <code>filter.accept(e) == true</code>.
     *
     * @return An <code>ItemIterator</code>.
     *
     * @throws RepositoryException If an unexpected error occurs.
     */
    public ItemIterator getItems(ItemFilter filter) throws RepositoryException;

    /**
     * Returns the deepest primary child item accessible via a chain of
     * primary child items from this node.
     * A node's type can specifiy a maximum of one of
     * its child items (child node or property) as its <i>primary child item</i>.
     * This method traverses the chain of primary child items of this node
     * until it either encounters a property or encounters a node that does not
     * have a primary child item. It then returns that property or node. If
     * this node itself (the one that this method is being called on) has no
     * primary child item then this method throws a
     * <code>ItemNotFoundException</code>.
     *
     * @throws ItemNotFoundException if this node does not have a primary
     * child item.
     *
     * @throws RepositoryException If another error occurs.
     *
     * @return the deepest primary child item accessible from this node via
     * a chain of primary child items.
     */
    public Item getPrimaryItem() throws ItemNotFoundException, RepositoryException;

    /**
     * Indicates whether a node exists at <code>path</code>
     * Returns <code>true</code> if a node exists at <code>path</code> and
     * <code>false</code> otherwise.
     *
     * @param path The path of a (possible) node.
     *
     * @return <code>true</code> if a node exists at <code>path</code>;
     * <code>false</code> otherwise.
     *
     * @throws RepositoryException If an unspecified error occurs.
     */
    public boolean hasNode(String path) throws RepositoryException;

    /**
     * Indicates whether a property exists at <code>path</code>
     * Returns <code>true</code> if a property exists at <code>path</code> and
     * <code>false</code> otherwise.
     *
     * @param path The path of a (possible) property.
     *
     * @return <code>true</code> if a property exists at <code>path</code>;
     * <code>false</code> otherwise.
     *
     * @throws RepositoryException If an unspecified error occurs.
     */
    public boolean hasProperty(String path) throws RepositoryException;

    /**
     * Indicates whether this node has child nodes.
     * Returns <code>true</code> if this node has one or more child nodes;
     * <code>false</code> otherwise.
     *
     * @return <code>true</code> if this node has one or more child nodes;
     * <code>false</code> otherwise.
     *
     * @throws RepositoryException If an unspecified error occurs.
     */
    public boolean hasNodes() throws RepositoryException;

    /**
     * Indicates whether this node has properties.
     * Returns <code>true</code> if this node has one or more properties;
     * <code>false</code> otherwise.
     *
     * @return <code>true</code> if this node has one or more properties;
     * <code>false</code> otherwise.
     *
     * @throws RepositoryException If an unspecified error occurs.
     */
    public boolean hasProperties() throws RepositoryException;

    /**
     * Returns the node type of this node.
     * The node type is assigned either explicitly or automatically upon node
     * creation. See {@link #addNode(String) addNode(String path)}
     * and {@link #addNode(String, String) addNode(String path, String nodeType)}
     *
     * @return a <code>NodeType</code> object.
     */
    public NodeType getNodeType();
    
    /**
     * Returns the UUID of this node.
     *
     * @return the UUID of this node
     */
    public String getUUID();

    /**
     * Indicates whether this node is of the specified node type.
     * Returns <code>true</code> if this node is of the specified node type
     * or a subtype of the specified node type. Returns <code>false</code> otherwise.
     * This method provides a quick method for determining whether
     * a particular node is of a particular type without having to
     * manually search the inheritance hierarchy (which, in some implementations
     * may be a multiple-inhertiance hierarchy, making a manual search even
     * more complex)
     *
     * @param nodeTypeName the name of a node type.
     *
     * @return true if this node is of the specified node type
     * or a subtype of the specified node type; returns false otherwise.
     *
     * @throws RepositoryException If an unspecified error occurs.
     */
    public boolean isNodeType(String nodeTypeName) throws RepositoryException;

    /**
     * Returns the definition of <i>this</i> <code>Node</code>. This method is
     * actually a shortcut to searching through this node's parent's node type
     * (and its supertypes) for the child node definition applicable to this
     * node.
     *
     * @see NodeType#getChildNodeDefs
     *
     * @return a <code>NodeDef</code> object.
     */
    public NodeDef getDefinition();

    /**
     * Creates a new version with a system generated version name and returns
     * that version.  If this node is not versionable, a
     * <code>UnsupportedRepositoryOperationException</code> is thrown.
     *
     * @throws UnsupportedRepositoryOperationException
     * @throws RepositoryException
     * @throws VersionException
     * @return a <code>Version</code> object.
     */
    public Version checkin() throws RepositoryException, UnsupportedRepositoryOperationException, VersionException;

    /**
     * Sets this versionable node to checked-out status by setting its
     * <code>jcr:isCheckedOut</code> property to <code>true</code>. If this
     * node is not versionable, a
     * <code>UnsupportedRepositoryOperationException</code> is thrown.
     *
     * @throws UnsupportedRepositoryOperationException
     * @throws RepositoryException
     */
    public void checkout() throws RepositoryException, UnsupportedRepositoryOperationException;

    /**
     * Returns <code>true</code> if this node is currently checked-out and
     * <code>false</code> otherwise.  If this node is not versionable, a
     * <code>UnsupportedRepositoryOperationException</code> is thrown.
     *
     * @throws UnsupportedRepositoryOperationException
     * @throws RepositoryException
     * @return a <code>boolean</code>.
     */
    public boolean isCheckedOut() throws RepositoryException, UnsupportedRepositoryOperationException;

    /**
     * Restores the version of this node with the specified version name.
     * Returns the node in its new restored state.  If this node is not versionable, a
     * <code>UnsupportedRepositoryOperationException</code> is thrown.
     *
     * @throws UnsupportedRepositoryOperationException
     * @throws RepositoryException
     * @return a <code>Node</code> object.
     */
    public Node restore(String versionName) throws RepositoryException, UnsupportedRepositoryOperationException;

    /**
     * Restores this node to the state specified by the given <code>Version</code> object.
     * If this node is not versionable, a
     * <code>UnsupportedRepositoryOperationException</code> is thrown.
     *
     * @throws UnsupportedRepositoryOperationException
     * @throws RepositoryException
     * @return a <code>Node</code> object.
     */
    public Node restore(Version version) throws RepositoryException, UnsupportedRepositoryOperationException;

    /**
     * Restores this node to the state specified by the given date.
     * If this node is not versionable, a
     * <code>UnsupportedRepositoryOperationException</code> is thrown.
     *
     * @throws UnsupportedRepositoryOperationException
     * @throws RepositoryException
     * @return a <code>Node</code> object.
     */
    public Node restore(Calendar date) throws RepositoryException, UnsupportedRepositoryOperationException;

    /**
     * Restores the version of this node with the specified version label.
     * Returns the node in its new restored state.  If this node is not versionable, a
     * <code>UnsupportedRepositoryOperationException</code> is thrown.
     *
     * @throws UnsupportedRepositoryOperationException
     * @throws RepositoryException
     * @return a <code>Node</code> object.
     */
    public Node restoreByLabel(String versionLabel) throws RepositoryException, UnsupportedRepositoryOperationException;

    /**
     * Returns the root version object of this node's version history. This is the dummy
     * version that is always present in a version history and represents the version
     * history as a whole. From here the entire version graph can be traversed uing the
     * methods of the <code>Version</code> interface.   If this node is not versionable, a
     * <code>UnsupportedRepositoryOperationException</code> is thrown.
     *
     * @throws UnsupportedRepositoryOperationException
     * @throws RepositoryException
     * @return a <code>Version</code> object.
     */
    public Version getRootVersion() throws RepositoryException, UnsupportedRepositoryOperationException;

    /**
     * Returns the current base version of this versionable node If this node is not versionable, a
     * <code>UnsupportedRepositoryOperationException</code> is thrown.
     *
     * @throws UnsupportedRepositoryOperationException
     * @throws RepositoryException
     * @return a <code>Version</code> object.
     */
    public Version getBaseVersion() throws RepositoryException, UnsupportedRepositoryOperationException;
}
