/*
 * PasswordCredentials.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr;

import java.io.Serializable;
import java.util.HashMap;

/**
 * A <code>SimpleCredentials</code> provides simple user ID/password
 * credentials.
 *
 * @author Stefan Guggisberg
 */
public final class SimpleCredentials implements Credentials, Serializable {

    private String userId;
    private char[] password;
    private HashMap attributes = new HashMap();

    /**
     * Create a new <code>SimpleCredentials</code> object, given a user ID
     * and password.
     * <p>
     * Note that the given user password is cloned before it is stored
     * in the new <code>SimpleCredentials</code> object. This should
     * avoid the risk of having unnecessary references to password data
     * lying around in memory.
     * <p>
     * @param userId the user ID
     * @param password the user's password
     */
    public SimpleCredentials(String userId, char[] password) {
        this.userId = userId;
        if (password != null) {
            this.password = (char[]) password.clone();
        }
       else {
            this.password = new char[] {};
        }
    }

    /**
     * Returns the user password.
     *
     * Note that this method returns a reference to the password.
     * It is the caller's responsibility to zero out the password information
     * after it is no longer needed.
     *
     * @return the password
     */
    public char[] getPassword() {
        return password;
    }

    /**
     * Gets the user ID.
     * <p>
     * <b>Level 1 and 2</b>
     * <p>
     * @return the user ID.
     */
    public String getUserId() {
        return userId;
    }

    /**
     * Stores an attribute in this credentials instance.
     * <p>
     * <b>Level 1 and 2</b>
     * <p>
     * @param name a <code>String</code> specifying the name of the attribute
     * @param value the <code>Object</code> to be stored
     */
    public void setAttribute(String name, Object value) {
        // name cannot be null
        if (name == null) {
            throw new IllegalArgumentException("name cannot be null");
        }

        // null value is the same as removeAttribute()
        if (value == null) {
            removeAttribute(name);
            return;
        }

        synchronized (attributes) {
            attributes.put(name, value);
        }
    }

    /**
     * Returns the value of the named attribute as an <code>Object</code>,
     * or <code>null</code> if no attribute of the given name exists.
     * <p>
     * <b>Level 1 and 2</b>
     * <p>
     * @param name a <code>String</code> specifying the name of
     *             the attribute
     *
     * @return	an <code>Object</code> containing the value
     *          of the attribute, or <code>null</code> if
     *          the attribute does not exist
     */
    public Object getAttribute(String name) {
        synchronized (attributes) {
            return (attributes.get(name));
        }
    }

    /**
     * Removes an attribute from this credentials instance.
     * <p>
     * <b>Level 1 and 2</b>
     * <p>
     * @param name a <code>String</code> specifying the name of the attribute
     *             to remove
     */
    public void removeAttribute(String name) {
        synchronized (attributes) {
            attributes.remove(name);
        }
    }

    /**
     * Returns the names of the attributes available to this
     * credentials instance. This method returns an empty array
     * if the credentials instance has no attributes available to it.
     * <p>
     * <b>Level 1 and 2</b>
     * <p>
     * @return a string array containing the names of the stored attributes
     */
    public String[] getAttributeNames() {
        synchronized (attributes) {
            return (String[]) attributes.keySet().toArray(new String[attributes.keySet().size()]);
        }
    }
}
