/*
 * PropertyDef.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr.nodetype;

import javax.jcr.Value;

/**
 * A property definition.
 *
 * @author Peeter Piegaze
 * @author Stefan Guggisberg
 */
public interface PropertyDef {

    /**
     * Gets the node type that contains the declaration of <i>this</i>
     * <code>PropertyDef</code>.
     *
     * @return a <code>NodeType</code> object.
     */
    public NodeType getDeclaringNodeType();

    /**
     * Gets the name of the property that this definition applies to. If
     * <code>null</code>, then this <code>PropertyDef</code> defines a residual
     * set of properties. That is, it defines the characteristics of all those
     * properties with names apart from the names explicitly used in other
     * property or child node definitions.
     *
     * @return A <code>String</code>, or <code>null</code>.
     */
    public String getName();

    /**
     * Gets the required type of the property. On of:
     *
     * <ul>
     *   <li><code>PropertyType.STRING</code></li>
     *   <li><code>PropertyType.DATE</code></li>
     *   <li><code>PropertyType.SOFTLINK</code></li>
     *   <li><code>PropertyType.BINARY</code></li>
     *   <li><code>PropertyType.DOUBLE</code></li>
     *   <li><code>PropertyType.LONG</code></li>
     *   <li><code>PropertyType.BOOLEAN</code></li>
     *   <li><code>PropertyType.UNDEFINED</code></li>
     * <ul>
     * If <code>PropertyType.UNDEFINED</code> then this property may be
     * of any type.
     *
     * @return an int
     */
    public int getRequiredType();

    /**
     * Gets the constraint string. This string describes the constraints
     * that exist on future values of the property.
     * <p>
     * Reporting of value constraints is <i>optional</i>. An implementation
     * is free to return <code>null</code> to this call, indicating that value
     * constraint information is unavailable (though a constraint may still
     * exist).
     * <p>
     * If a string is returned then it is interpreted in different ways
     * depending on the type specified for this property. The following
     * describes the value constraint syntax for each property type:
     *
     * <ul>
     *
     * <li><code>STRING</code>: The constraint string is a regular expression
     * pattern. For example the regular expression <b><code>".*"</code></b> means
     * "any string"</li>
     *
     * <li><code>SOFTLINK</code>: The constraint string is a comma separated list
     * of node type names. For example <code>"nt:authored, mynt:newsArticle"</code>.
     * Only soft links to nodes with at least one of the specifed object
     * classes can be assigned to the property. An empty string indicates that
     * then no constraint is placed on the SOFTLINK value, it may reference
     * nodes of any node type or repository locations where no node
     * currently exists. To constrain the value of the SOFTLINK value to point
     * only to existing nodes one can specify the constraint string
     * <code>"nt:base"</code>, which covers all possible node types.
     * A check on the existence and node type of the referenced node is
     * made whenever the value is set (on property creation or later) and the
     * modification is saved through <code>{@link javax.jcr.Node#save()}</code>.</li>
     *
     * <li><code>BOOLEAN</code>: Either <code>"true"</code> or
     * <code>"false"</code>.</li>
     *
     * </ul>
     *
     * The remaining types all have value constraints in the form of inclusive
     * or exclusive ranges: i.e., <code>"[<i>min</i>, <i>max</i>]"</code>,
     * <code>"(<i>min</i>, <i>max</i>)"</code>, <code>"(<i>min</i>, <i>max</i>]"</code> or
     * <code>"[<i>min</i>, <i>max</i>)"</code>. Where <code>"["</code> and <code>"]"</code>
     * indicate "inclusive", while <code>"("</code> and <code>")"</code>
     * indicate "exclusive". A missing <code><i>min</i></code> or <code><i>max</i></code> value
     * indicates no bound in that direction. For example <code>[,5]</code>
     * means no minimum but a maximum of 5 (inclusive).
     * The syntax and meaning of the <code><i>min</i></code> and <code><i>max</i></code> values
     * themselves differs between types as follows:
     *
     * <ul>
     *
     * <li><code>BINARY</code>: <code><i>min</i></code> and <code><i>max</i></code>
     * specify the allowed size range of the binary value in bytes.</li>
     *
     * <li><code>DATE</code>: <code><i>min</i></code> and <code><i>max</i></code>
     * are dates specifiying the allowed date range. The date strings must be
     * in the ISO8601-compliant format:
     * <code><i>YYYY-MM-DD</i>T<i>hh:mm:ss.sssTZD</i></code>.</li>
     *
     * <li><code>LONG</code>, <code>DOUBLE</code>: <code><i>min</i></code> and
     * <code><i>max</i></code> are numbers.</li>
     *
     * </ul>
     *
     * @return a <code>String</code> or <code>null</code>
     */
    public String getValueConstraint();

    /**
     * Gets the default value of the property. This is the value that the
     * property will be assigned if it is either automatically created, or
     * created without a specified initial value. If <code>null</code>,
     * then the property has no fixed default value. Note that this does not
     * exclude the possibility that the property still assumes some value
     * automatically, but that value may be variable (for example, "the current
     * date") and hence cannot be expressed as a single fixed value.
     *
     * @return a <code>Value</code> or <code>null</code>
     */
    public Value getDefaultValue();

    /**
     * Reports whether the property is to be automatically created when its
     * parent node is created. If <code>true</code> then this
     * <code>PropertyDef</code> will necessarily  not be a residual set
     * definition but will specify an actual property name (in other words
     * <code>getName()</code> will return a non-null value).
     *
     * @return a <code>boolean</code>
     */
    public boolean isAutoCreate();

    /**
     * Reports whether the property is mandatory. A mandatory property
     * is one which, if it exists, must have a value. Therefore, if this
     * attribute is set to <code>true</code>, the following restrictions
     * will necessarily be enforced:
     * <ul>
     * <li>
     *   If <code>autoCreate</code> is <code>true</code> then either
     *   <code>defaultValue</code> must also have a value, or the
     *   implementation must fill in this value. This does not require that the
     *   property be read-only, though it may be, in which case only the
     *   implementation, and not the client, can write to this property.
     * </li>
     * <li>
     *   If <code>autoCreate</code> is <code>false</code> then the client must
     *   ensure that this property is created (and if no default value is
     *   indicated, given a value) before the parent node is saved, otherwise
     *   a <code>ConstraintViolationException</code> will be thrown.
     * </li>
     * <li>
     *   Once created, this propertys value cannot be set to null.
     *   Nor can the property itself be removed without removing its parent node,
     *   otherwise a <code>ConstraintViolationException</code> is thrown on save
     *   of the parent node.
     * </li>
     * </ul>
     *
     * @return a <code>boolean</code>
     */
    public boolean isMandatory();

    /**
     * Gets the on-version status of the property. This governs what to do if
     * the parent node of this property is versioned; an
     * <code>OnParentVersionAction</code>:
     * <ul>
     *       <li><code>OnParentVersionAction#COPY</code>, or</li>
     *       <li><code>OnParentVersionAction#VERSION</code>, or</li>
     *       <li><code>OnParentVersionAction#INITIALIZE</code>, or</li>
     *       <li><code>OnParentVersionAction#COMPUTE</code>, or</li>
     *       <li><code>OnParentVersionAction#IGNORE</code>, or</li>
     *       <li><code>OnParentVersionAction#ABORT</code></li>
     * </ul>
     *
     * @see OnParentVersionAction
     *
     * @return an <code>int</code>
     */
    public int getOnVersion();

    /**
     * Reports whether the property is read-only. This means that this
     * property cannot be written-to via this API. It may be written to,
     * however, by the implementation itself via some mechanism not
     * specified by this standard.
     *
     * @return a <code>boolean</code>
     */
    public boolean isReadOnly();
    
    /**
     * Reports whether the property is multivalue.
     *
     * @return a <code>boolean</code>
     */
    public boolean isMultiple();

    /**
     * Reports whether this property is the primary child item of its parent
     * node. Since any given node may have either zero or one primary child
     * items, this means that a maximum of one <code>PropertyDef</code> or
     * <code>ChildNodeDef</code> within a particular <code>NodeType</code> may
     * return <code>true</code> on this call. The primary child item flag
     * is used by the method <code>{@link javax.jcr.Node#getPrimaryItem()}</code>.
     *
     * @return a <code>boolean</code>
     */
    public boolean isPrimaryItem();
}
