/*
 * QueryManager.java
 *
 * This file is part of the v0.12 specification for the
 * Java Content Repository API
 */
package javax.jcr.query;

import javax.jcr.InvalidPathException;

/**
 * This interface encapsulates methods for the management of search queries.
 * Provides methods for the creation and retrieval of search queries.
 *
 * @author Peeter Piegaze
 */
public interface QueryManager {

    /**
     * Creates an empty, transient <code>Query</code> object.
     *      
     * @param statement the query statement
     * @param language the query language
     *
     * @throws InvalidQueryException If the query <code>statement</code> is syntactically
     * invalid.
     * @throws UnsupportedQueryLanguageException If <code>language</code> is not
     * supported.
     *
     * @return A <code>Query</code> object.
     */
    public Query createQuery(String statement, int language) throws InvalidQueryException, UnsupportedQueryLanguageException;

    /**
     * Retrieves an existing persistent query. If <code>absPath</code> is not
     * the path of a valid persisted query (i.e., if it does not point to a
     * node of node type <code>nt:query</code>) then an
     * <code>RepositoryException</code> is thrown.
     *
     * @param absPath the absolute path of a persisted query.
     *
     * @throws RepositoryException If <code>absPath</code> is not the path of
     * a valid persisted query or if another error occurs.
     *
     * @return a <code>Query</code> object.
     */
    public Query getQuery(String absPath) throws InvalidPathException, InvalidQueryException;

    /**
     * Returns a string identifier for each supported query language. The
     * string should consist of two parts. The first identifying the main query
     * language, the second identifying the full-text search language embedded
     * in the <code>SEARCH</code> clause.
     * <p>
     * At minimum, the implementation must
     * support <code>JCRQL</code> with the <i>Simple Search-Engine Syntax</i>
     * (<code>SSES</code>) embedded in the <code>SEARCH</code> clause. The
     * string identifier for this language is <code>JCRQL-SSES</code>.
     * <p>
     * Optionally, an implementation may support other embedded full-text
     * languages and/or other main query languages. Possible examples might be
     * <code>JCRQL-Fulcrum_SearchSQL</code> and <code>XQuery</code>.
     *
     * @return An array of <code>QueryLanguage</code> object.
     */
    public QueryLanguage[] getSupportedQueryLanguages();
}
